@@ -2,13 +2,13 @@ import gc
 import itertools
 import os
 import ssl
+import uuid
 from copy import deepcopy
 from datetime import datetime, timedelta
 from pickle import dumps, loads
 from unittest.mock import Mock, patch
 
 import pytest
-from case import ContextMock, mock
 from vine import promise
 
 from celery import Celery, _state
@@ -16,6 +16,8 @@ from celery import app as _app
 from celery import current_app, shared_task
 from celery.app import base as _appbase
 from celery.app import defaults
+from celery.backends.base import Backend
+from celery.contrib.testing.mocks import ContextMock
 from celery.exceptions import ImproperlyConfigured
 from celery.loaders.base import unconfigured
 from celery.platforms import pyimplementation
@@ -23,6 +25,7 @@ from celery.utils.collections import DictAttribute
 from celery.utils.objects import Bunch
 from celery.utils.serialization import pickle
 from celery.utils.time import localize, timezone, to_utc
+from t.unit import conftest
 
 THIS_IS_A_KEY = 'this is a value'
 
@@ -218,6 +221,13 @@ class test_App:
         self.app._using_v1_reduce = True
         assert loads(dumps(self.app))
 
+    def test_autodiscover_tasks_force_fixup_fallback(self):
+        self.app.loader.autodiscover_tasks = Mock()
+        self.app.autodiscover_tasks([], force=True)
+        self.app.loader.autodiscover_tasks.assert_called_with(
+            [], 'tasks',
+        )
+
     def test_autodiscover_tasks_force(self):
         self.app.loader.autodiscover_tasks = Mock()
         self.app.autodiscover_tasks(['proj.A', 'proj.B'], force=True)
@@ -265,6 +275,14 @@ class test_App:
         with self.Celery(broker='foo://baribaz') as app:
             assert app.conf.broker_url == 'foo://baribaz'
 
+    def test_pending_configuration_non_true__kwargs(self):
+        with self.Celery(task_create_missing_queues=False) as app:
+            assert app.conf.task_create_missing_queues is False
+
+    def test_pending_configuration__kwargs(self):
+        with self.Celery(foo='bar') as app:
+            assert app.conf.foo == 'bar'
+
     def test_pending_configuration__setattr(self):
         with self.Celery(broker='foo://bar') as app:
             app.conf.task_default_delivery_mode = 44
@@ -494,6 +512,16 @@ class test_App:
         finally:
             _imports.MP_MAIN_FILE = None
 
+    def test_can_get_type_hints_for_tasks(self):
+        import typing
+
+        with self.Celery() as app:
+            @app.task
+            def foo(parameter: int) -> None:
+                pass
+
+            assert typing.get_type_hints(foo) == {'parameter': int, 'return': type(None)}
+
     def test_annotate_decorator(self):
         from celery.app.task import Task
 
@@ -556,20 +584,12 @@ class test_App:
         for key, value in changes.items():
             assert restored.conf[key] == value
 
-    # def test_worker_main(self):
-    #     from celery.bin import worker as worker_bin
-    #
-    #     class worker(worker_bin.worker):
-    #
-    #         def execute_from_commandline(self, argv):
-    #             return argv
-    #
-    #     prev, worker_bin.worker = worker_bin.worker, worker
-    #     try:
-    #         ret = self.app.worker_main(argv=['--version'])
-    #         assert ret == ['--version']
-    #     finally:
-    #         worker_bin.worker = prev
+    @patch('celery.bin.celery.celery')
+    def test_worker_main(self, mocked_celery):
+        self.app.worker_main(argv=['worker', '--help'])
+
+        mocked_celery.main.assert_called_with(
+            args=['worker', '--help'], standalone_mode=False)
 
     def test_config_from_envvar(self):
         os.environ['CELERYTEST_CONFIG_OBJECT'] = 't.unit.app.test_app'
@@ -752,6 +772,11 @@ class test_App:
         assert self.app.conf['FOO'] == 10
         assert self.app.conf['BAR'] == 20
 
+    @patch('celery.bin.celery.celery')
+    def test_start(self, mocked_celery):
+        self.app.start()
+        mocked_celery.main.assert_called()
+
     @pytest.mark.parametrize('url,expected_fields', [
         ('pyamqp://', {
             'hostname': 'localhost',
@@ -891,10 +916,10 @@ class test_App:
         assert 'add1' in self.app.conf.beat_schedule
         assert 'add2' in self.app.conf.beat_schedule
 
-    def test_pool_no_multiprocessing(self):
-        with mock.mask_modules('multiprocessing.util'):
-            pool = self.app.pool
-            assert pool is self.app._pool
+    @pytest.mark.masked_modules('multiprocessing.util')
+    def test_pool_no_multiprocessing(self, mask_modules):
+        pool = self.app.pool
+        assert pool is self.app._pool
 
     def test_bugreport(self):
         assert self.app.bugreport()
@@ -970,6 +995,63 @@ class test_App:
         app = CustomCelery(set_as_current=False)
         assert isinstance(app.tasks, TaskRegistry)
 
+    def test_oid(self):
+        # Test that oid is global value.
+        oid1 = self.app.oid
+        oid2 = self.app.oid
+        uuid.UUID(oid1)
+        uuid.UUID(oid2)
+        assert oid1 == oid2
+
+    def test_global_oid(self):
+        # Test that oid is global value also within threads
+        main_oid = self.app.oid
+        uuid.UUID(main_oid)
+        from concurrent.futures import ThreadPoolExecutor
+        with ThreadPoolExecutor(max_workers=1) as executor:
+            future = executor.submit(lambda: self.app.oid)
+        thread_oid = future.result()
+        uuid.UUID(thread_oid)
+        assert main_oid == thread_oid
+
+    def test_thread_oid(self):
+        # Test that thread_oid is global value in single thread.
+        oid1 = self.app.thread_oid
+        oid2 = self.app.thread_oid
+        uuid.UUID(oid1)
+        uuid.UUID(oid2)
+        assert oid1 == oid2
+
+    def test_backend(self):
+        # Test that app.bakend returns the same backend in single thread
+        backend1 = self.app.backend
+        backend2 = self.app.backend
+        assert isinstance(backend1, Backend)
+        assert isinstance(backend2, Backend)
+        assert backend1 is backend2
+
+    def test_thread_backend(self):
+        # Test that app.bakend returns the new backend for each thread
+        main_backend = self.app.backend
+        from concurrent.futures import ThreadPoolExecutor
+        with ThreadPoolExecutor(max_workers=1) as executor:
+            future = executor.submit(lambda: self.app.backend)
+        thread_backend = future.result()
+        assert isinstance(main_backend, Backend)
+        assert isinstance(thread_backend, Backend)
+        assert main_backend is not thread_backend
+
+    def test_thread_oid_is_local(self):
+        # Test that thread_oid is local to thread.
+        main_oid = self.app.thread_oid
+        uuid.UUID(main_oid)
+        from concurrent.futures import ThreadPoolExecutor
+        with ThreadPoolExecutor(max_workers=1) as executor:
+            future = executor.submit(lambda: self.app.thread_oid)
+        thread_oid = future.result()
+        uuid.UUID(thread_oid)
+        assert main_oid != thread_oid
+
 
 class test_defaults:
 
@@ -997,26 +1079,26 @@ class test_debugging_utils:
 class test_pyimplementation:
 
     def test_platform_python_implementation(self):
-        with mock.platform_pyimp(lambda: 'Xython'):
+        with conftest.platform_pyimp(lambda: 'Xython'):
             assert pyimplementation() == 'Xython'
 
     def test_platform_jython(self):
-        with mock.platform_pyimp():
-            with mock.sys_platform('java 1.6.51'):
+        with conftest.platform_pyimp():
+            with conftest.sys_platform('java 1.6.51'):
                 assert 'Jython' in pyimplementation()
 
     def test_platform_pypy(self):
-        with mock.platform_pyimp():
-            with mock.sys_platform('darwin'):
-                with mock.pypy_version((1, 4, 3)):
+        with conftest.platform_pyimp():
+            with conftest.sys_platform('darwin'):
+                with conftest.pypy_version((1, 4, 3)):
                     assert 'PyPy' in pyimplementation()
-                with mock.pypy_version((1, 4, 3, 'a4')):
+                with conftest.pypy_version((1, 4, 3, 'a4')):
                     assert 'PyPy' in pyimplementation()
 
     def test_platform_fallback(self):
-        with mock.platform_pyimp():
-            with mock.sys_platform('darwin'):
-                with mock.pypy_version():
+        with conftest.platform_pyimp():
+            with conftest.sys_platform('darwin'):
+                with conftest.pypy_version():
                     assert 'CPython' == pyimplementation()
 
 
