@@ -91,7 +91,7 @@ class test_Signature(CanvasCase):
         assert fun(*args) == x
 
     def test_replace(self):
-        x = Signature('TASK', ('A'), {})
+        x = Signature('TASK', ('A',), {})
         assert x.replace(args=('B',)).args == ('B',)
         assert x.replace(kwargs={'FOO': 'BAR'}).kwargs == {
             'FOO': 'BAR',
@@ -654,15 +654,19 @@ class test_group(CanvasCase):
         g1 = group(Mock(name='t1'), Mock(name='t2'), app=self.app)
         sig = Mock(name='sig')
         g1.link(sig)
+        # Only the first child signature of a group will be given the callback
+        # and it is cloned and made immutable to avoid passing results to it,
+        # since that first task can't pass along its siblings' return values
         g1.tasks[0].link.assert_called_with(sig.clone().set(immutable=True))
 
     def test_link_error(self):
         g1 = group(Mock(name='t1'), Mock(name='t2'), app=self.app)
         sig = Mock(name='sig')
         g1.link_error(sig)
-        g1.tasks[0].link_error.assert_called_with(
-            sig.clone().set(immutable=True),
-        )
+        # We expect that all group children will be given the errback to ensure
+        # it gets called
+        for child_sig in g1.tasks:
+            child_sig.link_error.assert_called_with(sig)
 
     def test_apply_empty(self):
         x = group(app=self.app)
@@ -1247,7 +1251,7 @@ class test_chord(CanvasCase):
         # When we freeze the chord, its body will be cloned and options set
         top_group.freeze()
         with subtests.test(
-            msg="Validate body group indicies count from 0 after freezing"
+            msg="Validate body group indices count from 0 after freezing"
         ):
             assert all(
                 embedded_body_elem is not body_elem
